#include <broken_edge.h>
#include <lemon/xy.h>
#include <math.h>

BrokenEdge::BrokenEdge(Gnome::Canvas::Group & g, Gnome::Canvas::Points p) : Line(g), isbutton(false)
{
  my_points=new Gnome::Art::Point[3];

  arrow=new Gnome::Canvas::Polygon(g);
  *arrow << Gnome::Canvas::Properties::fill_color("red");
  arrow->signal_event().connect(sigc::mem_fun(*this, &BrokenEdge::edge_former_event_handler));
  set_points(p);
}

BrokenEdge::~BrokenEdge()
{
  if(arrow)delete(arrow);
}

void BrokenEdge::set_points(Gnome::Canvas::Points p, bool move)
{
  bool set_arrow=false;
  if(!move)
    {
      if(p.size()==2)
	{
	  set_arrow=true;
	  Gnome::Canvas::Points points_with_center;
	  points_with_center.push_back(my_points[0]=p[0]);
	  points_with_center.push_back(my_points[1]=Gnome::Art::Point( (p[0].get_x()+p[1].get_x())/2+0 , (p[0].get_y()+p[1].get_y())/2 )+0 );
	  points_with_center.push_back(my_points[2]=p[1]);
	  property_points().set_value(points_with_center);
	}  
      if(p.size()==3)
	{
	  set_arrow=true;
	  property_points().set_value(p);
	  for(int i=0;i<3;i++)
	    {
	      my_points[i]=p[i];
	    }
	}
    }
  else
    {
      if(p.size()==2)
	{
	  Gnome::Canvas::Points points;
	  my_points[0]=p[0];
	  my_points[2]=p[1];
	  for(int i=0;i<3;i++)
	    {
	      points.push_back(my_points[i]);
	    }
	  property_points().set_value(points);
	}
    }

  if(set_arrow)
    {
      //calculating coordinates of the direction indicator arrow

      xy<gdouble> target( my_points[2].get_x(), my_points[2].get_y() );
      xy<gdouble> center( my_points[1].get_x(), my_points[1].get_y() );

      xy<gdouble> unit_vector_in_dir(target-center);
      //       std::cout << target << " - " << center << " = " << unit_vector_in_dir << "    / " <<unit_vector_in_dir.normSquare() ;
      unit_vector_in_dir/=sqrt( unit_vector_in_dir.normSquare() );
      //       std::cout << " = " << unit_vector_in_dir << std::endl;

      xy<gdouble> unit_norm_vector(0-unit_vector_in_dir.y, unit_vector_in_dir.x);
      //       std::cout << unit_norm_vector << std::endl;

      {      
	//       /\             top
	//      /  \
	//      -  -      c(enter)l(eft), ccl, ccr, cr
	//       ||
	//       ||       b(ottom)l, br
      }

      double size=3;

      xy<gdouble> bl (center - unit_vector_in_dir * 3 * size + unit_norm_vector * size );
      xy<gdouble> br (center - unit_vector_in_dir * 3 * size - unit_norm_vector * size );
      xy<gdouble> ccl(center + unit_vector_in_dir *  size + unit_norm_vector * size );
      xy<gdouble> ccr(center + unit_vector_in_dir *  size - unit_norm_vector * size );
      xy<gdouble> cl (center + unit_vector_in_dir *  size + unit_norm_vector * 2 * size );
      xy<gdouble> cr (center + unit_vector_in_dir *  size - unit_norm_vector * 2 * size );
      xy<gdouble> top(center + unit_vector_in_dir * 3 * size);
	 
      //       std::cout << bl << " " << br << " " << ccl << " "  << ccr << " " << cl << " " << cr << " " << top << std::endl;

      Gnome::Canvas::Points arrow_points;
      arrow_points.push_back(Gnome::Art::Point( bl.x , bl.y  ) );
      arrow_points.push_back(Gnome::Art::Point( br.x , br.y  ) );
      arrow_points.push_back(Gnome::Art::Point( ccr.x, ccr.y ) );
      arrow_points.push_back(Gnome::Art::Point( cr.x , cr.y  ) );
      arrow_points.push_back(Gnome::Art::Point( top.x, top.y ) );
      arrow_points.push_back(Gnome::Art::Point( cl.x , cl.y  ) );
      arrow_points.push_back(Gnome::Art::Point( ccl.x, ccl.y ) );

      arrow->property_points().set_value(arrow_points);
    }
}

bool BrokenEdge::edge_former_event_handler(GdkEvent* e)
{
  switch(e->type)
    {
    case GDK_BUTTON_PRESS:
      //we mark the location of the event to be able to calculate parameters of dragging
      clicked_x=e->button.x;
      clicked_y=e->button.y;
      isbutton=true;
      break;
    case GDK_BUTTON_RELEASE:
      isbutton=false;
      break;
    case GDK_MOTION_NOTIFY:
      //we only have to do sg. if the mouse button is pressed
      if(isbutton)
	{
	  //new coordinates will be the old values,
	  //because the item will be moved to the
	  //new coordinate therefore the new movement
	  //has to be calculated from here

	  double dx=e->motion.x-clicked_x;
	  double dy=e->motion.y-clicked_y;

	  Gnome::Canvas::Points points_new;

	  points_new.push_back(my_points[0]);
	  points_new.push_back(my_points[1]=Gnome::Art::Point(my_points[1].get_x()+dx,my_points[1].get_y()+dy));
	  points_new.push_back(my_points[2]);

	  set_points(points_new);

	  clicked_x=e->motion.x;
	  clicked_y=e->motion.y;

	}
    default: break;
    }

  return true;
}
